/******************************************************************/
/*            Conversion program for GDS2 to .geo format 	  */
/* Copyright (c) 1990 Sonnet Software, Inc., All rights reserved. */
/******************************************************************/

/* %W% last mod on %G% at %U% */
/* Full SCCS path name: (%P%) */

/*******************************************************************

	NAME: rec.c
	
	AUTHOR: K. Schroeder
	
	PURPOSE: Process each GDS2 record type
	
	CONTENTS:
	
*******************************************************************/

#include "local.h"
#include <stdio.h>
#include <string.h>
#include "gds.h"
#include "rec.h"

/* definitions	*/

#define TRACE_FORMAT(A) if (options.record_format) fprintf(trfile,A);

/* external	*/
extern struct options options;

/* structures and global/semiglobal variable */


/*******************************************************************
	NAME: cpdir
	
	DESCRIPTION: Copy all the files in directory source directory to 
	
	RETURNS: 

*******************************************************************/


void
rec_header(record,gds)
Record *record;
GDS *gds;
{
	int release;
	TRACE_FORMAT("<HEADER> ");

	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,2);

	release = (int)record->integer[0];
	switch (release) {
		case PRIOR_REL_3_0:
			print_gds_version("Before Release 3.0",release);
			DEBUG2(trfile,"Before Release 3.0\n");
			gds->release = PRIOR_REL_3_0;
			break;
			
		case REL_3_0:
			print_gds_version("Release 3.0",-1);
			DEBUG2(trfile,"Release = 3.0\n");
			gds->release = REL_3_0;
			break;
			
		case REL_4_0:
			print_gds_version("Release 4.0",-1);
			DEBUG2(trfile,"Release = 4.0\n");
			gds->release = REL_4_0;
			break;
			
		case REL_5_0:
			print_gds_version("Release 5.0",-1);
			DEBUG2(trfile,"Release = 5.0\n");
			gds->release = REL_5_0;
			break;
			
		case REL_6_0:
			print_gds_version("Release 6.0",-1);
			DEBUG2(trfile,"Release = 6.0\n");
			gds->release = REL_6_0;
			break;
			
		case REL_6_0_1:
			print_gds_version("Release 6.0.1",-1);
			DEBUG2(trfile,"Release = 6.0.1\n");
			gds->release = REL_6_0_1;
			break;
			
		case REL_6_0_2:
			print_gds_version("Release 6.0.2",-1);
			DEBUG2(trfile,"Release = 6.0.2\n");
			gds->release = REL_6_0_2;
			break;

		case REL_6_1_0:
			print_gds_version("Release 6.1.0",-1);
			DEBUG2(trfile,"Release = 6.1.0\n");
			gds->release = REL_6_1_0;
			break;
			
			
		default:
			print_gds_version("Unknown release number", release);
			WARNING(stderr,"Unknown release number = %d\n",
				record->integer[0]);
			WARNING(stderr,"Will try to convert file anyway.\n");
			break;
	}
}

LOCAL_FN void
print_gds_version(rel_str, release)
char *rel_str;
int release;
{
	if ( options.gds_version ) {
		printf("%s",rel_str);
		if ( release == -1 ) {
			printf("\n");
		} else {
			printf(", Release = %d\n",release);
		}
		exit (0);
	}
}

void
rec_bgnlib(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<BGNLIB> ");
	DEBUG2(trfile,"Library time stamp\n");
	print_date(record,gds);
}

void
rec_libname(record,gds)
Record *record;
GDS *gds;
{
	char *name;

	TRACE_FORMAT("<LIBNAME> ");
	name = (char *)record->ascii;
	print_string("Library name",record,gds);
	if ( options.list_structures ) {
		printf("Library: %s\n",name);
	}
}

#define SET_UNIT(value,name) \
	if ( unit < value + value/1000.0 && unit > value - value/1000.0 ) { \
		gds->unit_name = name; \
		gds->conv = value; \
		DEBUG2(trfile,"Data base unit: %s, value = %g\n", gds->unit_name,gds->conv); \
		return; \
	}

void
rec_units(record,gds)
Record *record;
GDS *gds;
{ 
	double unit;	/* units per meter */
	TRACE_FORMAT("<UNITS>\n");

	check_record_type(record,gds,EIGHT_BYTE_REAL);
	check_record_length(record,gds,16);
	DEBUG2(trfile,"Data base unit in user unit: %g\n", record->real[0] );
	DEBUG2(trfile,"Data base unit in meters: %g\n", record->real[1] );
	gds->user_units = record->real[0];
	gds->db_units = record->real[1];
	unit = gds->db_units / gds->user_units;

	SET_UNIT( 0.0000254, "mils" );
	SET_UNIT( 0.0254, "inches" );
	SET_UNIT( 0.000001, "uM" );
	SET_UNIT( 0.001, "mm" );
	SET_UNIT( 0.01, "cm" );
	SET_UNIT( 1.0, "M" );

	/* ELSE */
	gds->unit_name = "units";
	gds->conv = unit;
	DEBUG2(trfile,"Data base unit: %s, value = %g\n", gds->unit_name,gds->conv);

}

void
rec_endlib(record,gds)
Record *record;
GDS *gds;
{
	gds->endlib = TRUE;
	TRACE_FORMAT("<ENDLIB>\n");
	NO_DATA_PRESENT("End of Library\n");
}

void
rec_bgnstr(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<BGNSTR> ");
	DEBUG2(trfile,"Structure time stamp\n");
	format_error(gds->struct_lib==TRUE, record ,gds);
	print_date(record,gds);
	gds->struct_lib = TRUE;
	
}

void
rec_strname(record,gds)
Record *record;
GDS *gds;
{
	Structure *add_structure();
	char *name;
	
	TRACE_FORMAT("<STRNAME> - \"");

	print_string("Structure name",record,gds);
	name = (char *)record->ascii;

	TRACE_FORMAT(name);
	TRACE_FORMAT("\"\n");

	if ( options.list_structures ) {
		printf("%s\n",name);
	} else {
		if ( structure_to_print == NULL ) { 
			structure_to_print = new( (strlen(name)+1)   *sizeof(char) );
			strcpy(structure_to_print,name);
		}
	}
	gds->structure = add_structure(gds,name);
	record->ascii = NULL;
}

void
rec_endstr(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<ENDSTR>\n");
	
	NO_DATA_PRESENT("End of Structure\n");
	gds->structure = NULL;
	gds->struct_lib = FALSE;
}

void
rec_boundary(record,gds)
Record *record;
GDS *gds;
{

	TRACE_FORMAT("<BOUNDRY> ");
	
	NO_DATA_PRESENT("Beginning of a Boundary\n");

	/* gds->structure is defined in STRNAME */
	format_error(gds->struct_lib==FALSE, record ,gds);
	format_error( gds->structure == NULL, record ,gds);
	add_element( gds->structure,BOUNDARY,gds->record_nu);
	gds->record_typ = BOUNDARY;
}

void
rec_path(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<PATH> ");
	
	NO_DATA_PRESENT("Beginning of a Path\n");

	/* gds->structure is defined in STRNAME */
	format_error(gds->struct_lib==FALSE, record ,gds);
	format_error(gds->structure==NULL, record ,gds);
	add_element( gds->structure,PATH,gds->record_nu);
	gds->record_typ = PATH;
}

void
rec_sref(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<SREF> ");
	
	NO_DATA_PRESENT("Beginning of a Structure Reference\n");
	format_error(gds->struct_lib==FALSE, record ,gds);
	format_error(gds->structure==NULL, record ,gds);
	add_element( gds->structure,SREF,gds->record_nu);
	gds->record_typ = SREF;

}

void
rec_aref(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<AREF> ");
	
	NO_DATA_PRESENT("Beginning of a Array Reference\n");

	format_error(gds->struct_lib==FALSE, record ,gds);
	format_error(gds->structure==NULL, record ,gds);
	add_element( gds->structure,AREF,gds->record_nu);
	gds->record_typ = AREF;
}

void
rec_text(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<TEXT> ");
	
	NO_DATA_PRESENT("Beginning of a Array Text\n");
	gds->record_typ = TEXTTYPE;
	SKIP;
}

void
rec_layer(record,gds)
Record *record;
GDS *gds;
{
	Element *element;
	Boundry *boundry;
	Path *path;
	int layer;
	
	TRACE_FORMAT("<LAYER> ");
	
	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,2);
	layer = (int)record->integer[0];
	
	DEBUG2(trfile,"Layer is %d\n", layer );
	
	element = get_element(gds);
	switch (element->typ) {
		case BOUNDARY:
			boundry = (Boundry *)element->desc;
			boundry->layer = layer;
			break;
		case PATH:
			path = (Path *)element->desc;
			path->layer = layer;
			break;
		default:
			break;
	}
}

void
rec_datatype(record,gds)
Record *record;
GDS *gds;
{
	Element *element;
	Boundry *boundry;
	Path *path;
	int datatype;
	
	TRACE_FORMAT("<DATATYPE> ");
	
	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,2);
	datatype = (int)record->integer[0];
	
	DEBUG2(trfile,"Date type is %d\n", (int)record->integer[0] );
	
	element = get_element(gds);
	switch (element->typ) {
		case BOUNDARY:
			boundry = (Boundry *)element->desc;
			boundry->datatype = datatype;
			break;
		case PATH:
			path = (Path *)element->desc;
			path->datatype = datatype;
			break;
		default:
			break;
	}
}

void
rec_width(record,gds)
Record *record;
GDS *gds;
{
	int width;
	Element *element;
	Path *path;
	
	TRACE_FORMAT("<WIDTH> ");
	
	check_record_type(record,gds,FOUR_BYTE_INT);
	check_record_length(record,gds,4);
	width = (int)record->integer[0];
	
	DEBUG2(trfile,"Width is %f\n", width * gds->user_units );
	
	element = get_element(gds);
	if ( gds->record_typ == PATH ) {
		path = (Path *)element->desc;
		path->width = width;
	} else if ( gds->record_typ != TEXTTYPE ) {
		WARNING(stderr,"PATHTYPE is defined in %s record, record number = %ld\n",
			print_record_type(element->typ), gds->record_nu);
	}
	
}

void
rec_endel(record,gds)
Record *record;
GDS *gds;
{

	Element *element;
	Boundry *boundry;
	Sref *sref;
	Aref *aref;
	Path *path;

	TRACE_FORMAT("<ENDEL>\n");
	
	NO_DATA_PRESENT("End of Element\n");

	element = get_element(gds);
	
	switch (element->typ) {
		case BOUNDARY:
			boundry = (Boundry *)element->desc;
			format_error(boundry==NULL, record ,gds);
			format_error(boundry->point==NULL, record ,gds);
			break;
		case PATH:
			path = (Path *)element->desc;
			format_error(path==NULL, record ,gds);
			format_error(path->point==NULL, record ,gds);
			break;
		case SREF:
			sref = (Sref *)element->desc;
			format_error(sref==NULL, record ,gds);
			format_error(sref->xtfm.xy_defined==FALSE, record ,gds);
			break;
		case AREF:
			aref = (Aref *)element->desc;
			format_error(aref==NULL, record ,gds);
			format_error(aref->xtfm.xy_defined==FALSE, record ,gds);
			break;
	}

	element_undefined();
}

void
rec_sname(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<SNAME> \"");
	TRACE_FORMAT(record->ascii);
	TRACE_FORMAT("\" " );

	format_error(gds->structure==NULL, record ,gds);
	print_string("Structure reference name",record,gds);
	sname(gds, record->ascii );
        record->ascii = NULL;
}

void
rec_xy(record,gds)
Record *record;
GDS *gds;
{
	Element *element;
	Boundry *boundry;
	Sref *sref;
	Aref *aref;
	Path *path;
	int num;
	
	TRACE_FORMAT("<XY> ");
	
	/* Unit record must have been defined */
	if ( gds->user_units == -1.0 ) {
		ERROR(stderr,"Units record not defined in GDSII file %s\n",gds->filename);
		quit(1);
	}
	
	format_error(gds->struct_lib==FALSE, record ,gds);
	check_record_type(record,gds,FOUR_BYTE_INT);
#ifdef ALPHA 
	num = record->length/4;
#else
	num = record->length/sizeof(long);
#endif
	element = get_element(gds);
	
	switch (element->typ) {
		case BOUNDARY:
			boundry = (Boundry *)element->desc;
			format_error(boundry->layer==NOT_DEFINED, record ,gds);
			format_error(boundry->datatype==NOT_DEFINED, record ,gds);
			boundry->point = create_point(record->integer,num);
			if ( debug >= 3 ) {
				dump_integer(num,record->integer,gds->user_units);
			}
			record->integer = NULL;
			break;
		case PATH:
			path = (Path *)element->desc;
			path->point = create_point(record->integer,num);
			format_error(path->layer==NOT_DEFINED, record ,gds);
			format_error(path->datatype==NOT_DEFINED, record ,gds);
			if ( debug >= 3 ) {
				dump_integer(num,record->integer,gds->user_units);
			}
			record->integer = NULL;
			break;
		case SREF:
			sref = (Sref *)element->desc;
			sref->xtfm.xy_defined = TRUE;
			sref->xtfm.x = record->integer[0];
			sref->xtfm.y = record->integer[1];
			if ( debug >= 3 ) {
				dump_integer(num,record->integer,gds->user_units);
			}
			break;
		case AREF:
			aref = (Aref *)element->desc;
			aref->xtfm.xy_defined = TRUE;
			aref->xtfm.x = record->integer[0];
			aref->xtfm.y = record->integer[1];
			aref->col_x  = record->integer[2];
			aref->col_y  = record->integer[3];
			aref->row_x  = record->integer[4];
			aref->row_y  = record->integer[5];
			if ( debug >= 3 ) {
				dump_integer(num,record->integer,gds->user_units);
			}
			break;
		default:
			break;
	}
}

void
rec_colrow(record,gds)
Record *record;
GDS *gds;
{
	Element *element;
	Aref *aref;
	
	TRACE_FORMAT("<COLROW> ");

	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,4);
	element = get_element(gds);
	format_error(element==NULL, record ,gds);
	aref = (Aref *)element->desc;
	DEBUG2(trfile,"Number of columns: %d\n", (int)record->integer[0] );
	DEBUG2(trfile,"Number of rows: %d\n", (int)record->integer[1] );
	aref->col = (int)record->integer[0];
	aref->row = (int)record->integer[1];
}

void
rec_textnode(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<TEXTNODE> ");
	
	NO_DATA_PRESENT("Beginning of a Text Node\n");
	SKIP;
}

void
rec_node(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<NODE> ");
	
	NO_DATA_PRESENT("Beginning of a Node\n");
}

void
rec_texttype(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<TEXTTYPE> ");
	
	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,2);
	SKIP;
}

void
rec_presentation(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<PRESENTATION> ");
	
	check_record_type(record,gds,BIT_ARRAY);
	SKIP;
}

void
rec_spacing()
{
	TRACE_FORMAT("<SPACING> ");
	DISCONTINUED;
}

void
rec_string(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<STRING> ");
	
	check_record_type(record,gds,ASCII_STRING);
	SKIP;
}

void
rec_strans(record,gds)
Record *record;
GDS *gds;
{
	int reflection;
	int abs_mag;
	int abs_angle;
	unsigned char *content;
	Element *element;
	Sref *sref;
	Aref *aref;

	TRACE_FORMAT("<STRANS> ");
	
	content = record->content;
	check_record_type(record,gds,BIT_ARRAY);
	check_record_length(record,gds,2);
	reflection = isbitset( content[0],7);
	abs_mag =    isbitset( content[1],2);
	abs_angle =  isbitset( content[1],1);

	element = get_element(gds);
	switch ( element->typ ) {
		case SREF:
			sref = (Sref *)element->desc;
			format_error(sref==NULL, record ,gds);
			sref->xtfm.reflection = reflection;
			sref->xtfm.abs_angle = abs_angle;
			sref->xtfm.abs_mag = abs_mag;
			break;
		case AREF:
			aref = (Aref *)element->desc;
			format_error(aref==NULL, record ,gds);
			aref->xtfm.reflection = reflection;
			aref->xtfm.abs_angle = abs_angle;
			aref->xtfm.abs_mag = abs_mag;
			break;
		default:
			break;
	}
	
	DEBUG2(trfile,"Reflection is %s\n",issetstr(reflection) );
	DEBUG2(trfile,"Absolute magnification is %s\n",issetstr(abs_mag) );
	DEBUG2(trfile,"Absolute angle is %s\n",issetstr(abs_angle) );
}

void
rec_mag(record,gds)
Record *record;
GDS *gds;
{
	Sref *sref;
	Aref *aref;
	double mag;
	Element *element;
	
	TRACE_FORMAT("<MAG> ");
	
	check_record_type(record,gds,EIGHT_BYTE_REAL);
	check_record_length(record,gds,8);
	mag = record->real[0];
	DEBUG2(trfile,"Magnification is %g\n", mag );
	
	element = get_element(gds);
	switch ( element->typ ) {
		case SREF:
			sref = (Sref *)element->desc;
			format_error(sref==NULL, record ,gds);
			sref->xtfm.mag = mag;
			break;
		case AREF:
			aref = (Aref *)element->desc;
			format_error(aref==NULL, record ,gds);
			aref->xtfm.mag = mag;
			break;
		default:
			break;
	}
}

void
rec_angle(record,gds)
Record *record;
GDS *gds;
{
	Sref *sref;
	Aref *aref;
	Element *element;
	double angle;

	TRACE_FORMAT("<ANGLE> ");
	
	check_record_type(record,gds,EIGHT_BYTE_REAL);
	check_record_length(record,gds,8);
	angle = record->real[0];
	DEBUG2(trfile,"Angle is %g\n", angle );

	element = get_element(gds);
	switch ( element->typ ) {
		case SREF:
			sref = (Sref *)element->desc;
			format_error(sref==NULL, record ,gds);
			sref->xtfm.angle = angle;
			break;
		case AREF:
			aref = (Aref *)element->desc;
			format_error(aref==NULL, record ,gds);
			aref->xtfm.angle = angle;
			break;
		default:
			break;
	}
}

void
rec_uinteger()
{
	TRACE_FORMAT("<UINTEGER> ");
	DISCONTINUED;
}

void
rec_ustring()
{
	TRACE_FORMAT("<USTRING> ");
	DISCONTINUED;
}

void
rec_reflibs(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<REFLIBS> ");
	
	print_string("Reference library name",record,gds);
}

void
rec_fonts()
{
	TRACE_FORMAT("<FONTS> ");
	SKIP;
}

void
rec_pathtype(record,gds)
Record *record;
GDS *gds;
{
	Element *element;
	Path *path;
	int pathtype;
	
	TRACE_FORMAT("<PATHTYPE> ");
	
	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,2);
	
	pathtype = (int)record->integer[0];
	
	/* make sure path is valid value */
	if ( pathtype != PATHTYPE_0 &&
	     pathtype != PATHTYPE_1 &&
	     pathtype != PATHTYPE_2 ){
		ERROR(stderr,"Pathtype is invalid value, pathtype = %d\n",pathtype);
		ERROR(stderr,"Error occured in record number = %ld\n", gds->record_nu);
		quit (1);
	}
	DEBUG2(trfile,"Path type is %d\n", pathtype );
	
	element = get_element(gds);
	if ( gds->record_typ == PATH ) {
		path = (Path *)element->desc;
		format_error(path==NULL, record ,gds);
		path->pathtype = pathtype;

	} else if ( gds->record_typ != TEXTTYPE ) {
		WARNING(stderr,"PATHTYPE is defined in %s record, record number = %ld\n",
			print_record_type(element->typ), gds->record_nu);
	}
}

void
rec_generations(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<GENERATIONS> ");
	
	check_record_length(record,gds,2);
	check_record_type(record,gds,TWO_BYTE_INT);
	DEBUG2(trfile,"Number of copies of deleted or backup-up structures: %d\n",
		record->integer[0]);
}

void
rec_attrtable(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<ATTRTABLE> ");
	print_string("Attribute definition file name",record,gds);
}

void
rec_styptable()
{
	TRACE_FORMAT("<STYPTABLE> ");
	DISCONTINUED;
}

void
rec_strtype()
{
	TRACE_FORMAT("<STRTYPE> ");
	DISCONTINUED;
}

void
rec_elflags(record,gds)
Record *record;
GDS *gds;
{
	int external_data;
	int template_data;
	unsigned char *content;

	TRACE_FORMAT("<ELFLAGS> ");
	
	content = record->content;
	check_record_type(record,gds,BIT_ARRAY);
	check_record_length(record,gds,2);
	external_data  = isbitset( content[1],1);
	template_data = isbitset( content[1],0);

	DEBUG2(trfile,"External data is %s\n",issetstr(external_data) );
	DEBUG2(trfile,"Template data is %s\n",issetstr(template_data) );
}

void
rec_elkey()
{
	TRACE_FORMAT("<ELKEY> ");
	
	DISCONTINUED;
}

void
rec_linktype()
{
	TRACE_FORMAT("<LINKTYPE> ");
	DISCONTINUED;
}

void
rec_linkkeys()
{
	TRACE_FORMAT("<LINKKEYS> ");
	DISCONTINUED;
}

void
rec_nodetype(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<NODETYPE> ");
	
	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,2);
	DEBUG2(trfile,"Node type is %d\n", (int)record->integer[0] );
	SKIP;
}

void
rec_popattr()
{
	TRACE_FORMAT("<POPATTR> ");
	
	SKIP;
}

void
rec_propvalue()
{
	TRACE_FORMAT("<PROPVALUE> ");
	SKIP;
}

void
rec_box(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<BOX> ");
	
	NO_DATA_PRESENT("Beginning of a Box\n");
	SKIP;
}

void
rec_boxtype(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<BOXTYPE> ");
	
	check_record_type(record,gds,TWO_BYTE_INT);
	check_record_length(record,gds,2);
	DEBUG2(trfile,"Box type is %d\n", (int)record->integer[0] );
}

void
rec_plex(record,gds)
Record *record;
GDS *gds;
{
	TRACE_FORMAT("<PLEX> ");
	
	check_record_type(record,gds,FOUR_BYTE_INT);
	SKIP;
}

void
rec_bgnextn()
{
	TRACE_FORMAT("<BGNEXTN> ");
	
	SKIP;
}

void
rec_endextn()
{
	TRACE_FORMAT("<ENDEXTN> ");
	SKIP;
}

void
rec_tapenum()
{
	TRACE_FORMAT("<TAPENUM> ");
	SKIP;
}

void
rec_tapecode()
{
	TRACE_FORMAT("<TAPECODE> ");
	SKIP;
}

void
rec_strclass()
{
	TRACE_FORMAT("<STRCLASS> ");
	SKIP;
}

void
rec_reserved()
{
	TRACE_FORMAT("<RESERVED> ");
	SKIP;
}

void
rec_format()
{
	TRACE_FORMAT("<FORMAT> ");
	SKIP;
}

void
rec_mask()
{
	TRACE_FORMAT("<MASK> ");
	SKIP;
}

void
rec_endmasks()
{
	TRACE_FORMAT("<ENDMASK>\n");
	SKIP;
}

void
rec_libdirsize()
{
	TRACE_FORMAT("<LIBDIRSIZE> ");
	SKIP;
}

void
rec_srfname()
{
	TRACE_FORMAT("<SRFNAME> ");
	SKIP;
}

void
rec_libsecur()
{
	TRACE_FORMAT("<LIBSECUR> ");
	SKIP;
}

/*******************************************************************
	NAME: format_error
	
	DESCRIPTION: If bad_format is TRUE then the record that was
		read was unexpected.  The expected record is 
		"expected_record_type".  If bad_format is TRUE,
		print error message then exit.
	
	RETURNS: none

*******************************************************************/

LOCAL_FN void
format_error(bad_format, record, gds)
int bad_format;		/* If TRUE then record format is bad */
Record *record;
GDS *gds;
{
	if ( bad_format ) {
		ERROR(stderr,"Unexpected record type %s encountered in record %ld, file=%s\n",
			print_record_type(record->rec_type),
			gds->record_nu,
			gds->filename);
		quit(1);
	}
}
